@@ -59,6 +59,7 @@ typedef enum {
 	NAUTILUS_FILE_SORT_BY_SIZE,
 	NAUTILUS_FILE_SORT_BY_TYPE,
 	NAUTILUS_FILE_SORT_BY_MTIME,
+        NAUTILUS_FILE_SORT_BY_ATIME,
 	NAUTILUS_FILE_SORT_BY_EMBLEMS
 } NautilusFileSortType;	
 
@@ -179,6 +180,7 @@ void                    nautilus_file_set_keywords                      (Nautilu
 GList *                 nautilus_file_get_emblem_names                  (NautilusFile                   *file);
 char *                  nautilus_file_get_top_left_text                 (NautilusFile                   *file);
 char *                  nautilus_file_peek_top_left_text                (NautilusFile                   *file,
+									 gboolean                        need_large_text,
 									 gboolean                       *got_top_left_text);
 gboolean                nautilus_file_get_directory_item_mime_types     (NautilusFile                   *file,
 									 GList                         **mime_list);
