@@ -54,6 +54,9 @@ typedef struct {
 	/* Whether this item was selected before rubberbanding. */
 	eel_boolean_bit was_selected_before_rubberband : 1;
 
+	/* Whether this item is visible in the view. */
+	eel_boolean_bit is_visible : 1;
+
 	/* Whether a monitor was set on this icon. */
 	eel_boolean_bit is_monitored : 1;
 
