@@ -40,6 +40,7 @@
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-bookmarks.h"
 #include "nautilus-zoom-control.h"
+#include "nautilus-search-bar.h"
 #include <eel/eel-debug.h>
 #include <eel/eel-marshal.h>
 #include <eel/eel-gdk-extensions.h>
@@ -58,7 +59,7 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkvbox.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-messagebox.h>
@@ -77,6 +78,7 @@
 #include <libnautilus-private/nautilus-view-factory.h>
 #include <libnautilus-private/nautilus-clipboard.h>
 #include <libnautilus-private/nautilus-undo.h>
+#include <libnautilus-private/nautilus-search-directory.h>
 #include <math.h>
 #include <sys/time.h>
 
@@ -88,6 +90,8 @@
 
 #define MAX_HISTORY_ITEMS 50
 
+#define EXTRA_VIEW_WIDGETS_BACKGROUND "#a7c6e1"
+
 /* FIXME bugzilla.gnome.org 41245: hardwired sizes */
 #define SIDE_PANE_MINIMUM_WIDTH 1
 #define SIDE_PANE_MINIMUM_HEIGHT 400
@@ -142,7 +146,7 @@ nautilus_window_init (NautilusWindow *window)
 	GtkWidget *table;
 	GtkWidget *menu;
 	GtkWidget *statusbar;
-      
+
 	window->details = g_new0 (NautilusWindowDetails, 1);
 
 	window->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT;
@@ -150,18 +154,17 @@ nautilus_window_init (NautilusWindow *window)
 	/* Set initial window title */
 	gtk_window_set_title (GTK_WINDOW (window), _("Nautilus"));
 
-	table = gtk_table_new (1, 5, FALSE);
+	table = gtk_table_new (1, 6, FALSE);
 	window->details->table = table;
 	gtk_widget_show (table);
 	gtk_container_add (GTK_CONTAINER (window), table);
 
-
 	statusbar = gtk_statusbar_new ();
 	window->details->statusbar = statusbar;
 	gtk_table_attach (GTK_TABLE (table),
 			  statusbar,
 			  /* X direction */                   /* Y direction */
-			  0, 1,                               4, 5,
+			  0, 1,                               5, 6,
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0,
 			  0,                                  0);
 	window->details->help_message_cid = gtk_statusbar_get_context_id
@@ -180,7 +183,6 @@ nautilus_window_init (NautilusWindow *window)
 			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0,
 			  0,                                  0);
 
-	
 	/* Register IconFactory callback to update the window border icon
 	 * when the icon-theme is changed.
 	 */
@@ -200,6 +202,8 @@ nautilus_window_init (NautilusWindow *window)
 static void
 nautilus_window_ui_update (NautilusWindow *window)
 {
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
 	gtk_ui_manager_ensure_update (window->details->ui_manager);
 }
 
@@ -218,6 +222,8 @@ nautilus_window_clear_status (gpointer callback_data)
 void
 nautilus_window_set_status (NautilusWindow *window, const char *text)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	if (text != NULL && text[0] != '\0') {
 		gtk_statusbar_push (GTK_STATUSBAR (window->details->statusbar), 0, text);
 	} else {
@@ -228,6 +234,8 @@ nautilus_window_set_status (NautilusWindow *window, const char *text)
 void
 nautilus_window_go_to (NautilusWindow *window, const char *uri)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	nautilus_window_open_location (window, uri, FALSE);
 }
 
@@ -235,6 +243,8 @@ nautilus_window_go_to (NautilusWindow *window, const char *uri)
 void
 nautilus_window_go_to_with_selection (NautilusWindow *window, const char *uri, GList *new_selection)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	nautilus_window_open_location_with_selection (window, uri, new_selection, FALSE);
 }
 
@@ -254,7 +264,9 @@ nautilus_window_go_up (NautilusWindow *window, gboolean close_behind)
 	GnomeVFSURI *parent_uri;
 	GList *selection;
 	char *parent_uri_string;
-	
+
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	if (window->details->location == NULL) {
 		return;
 	}
@@ -297,6 +309,8 @@ real_set_allow_up (NautilusWindow *window,
 void
 nautilus_window_allow_up (NautilusWindow *window, gboolean allow)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
 			 set_allow_up, (window, allow));
 }
@@ -356,6 +370,8 @@ nautilus_window_go_home (NautilusWindow *window)
 {
 	char *home_uri;
 
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
 	
 	g_assert (home_uri != NULL);
@@ -364,12 +380,13 @@ nautilus_window_go_home (NautilusWindow *window)
 }
 
 void
-nautilus_window_prompt_for_location (NautilusWindow *window)
+nautilus_window_prompt_for_location (NautilusWindow *window,
+				     const char     *initial)
 {
-	g_assert (NAUTILUS_IS_WINDOW (window));
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 	
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
-                         prompt_for_location, (window));
+                         prompt_for_location, (window, initial));
 }
 
 char *
@@ -380,9 +397,25 @@ nautilus_window_get_location (NautilusWindow *window)
 	return g_strdup (window->details->location);
 }
 
+void
+nautilus_window_set_search_mode (NautilusWindow *window,
+				 gboolean search_mode,
+				 NautilusSearchDirectory *search_directory)
+{
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
+	window->details->search_mode = search_mode;
+
+	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
+			 set_search_mode, (window, search_mode, search_directory));
+}
+
+
 void
 nautilus_window_zoom_in (NautilusWindow *window)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	if (window->content_view != NULL) {
 		nautilus_view_bump_zoom_level (window->content_view, 1);
 	}
@@ -400,6 +433,8 @@ nautilus_window_zoom_to_level (NautilusWindow *window,
 void
 nautilus_window_zoom_out (NautilusWindow *window)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	if (window->content_view != NULL) {
 		nautilus_view_bump_zoom_level (window->content_view, -1);
 	}
@@ -408,6 +443,8 @@ nautilus_window_zoom_out (NautilusWindow *window)
 void
 nautilus_window_zoom_to_default (NautilusWindow *window)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	if (window->content_view != NULL) {
 		nautilus_view_restore_default_zoom_level (window->content_view);
 	}
@@ -490,6 +527,8 @@ nautilus_window_set_initial_window_geometry (NautilusWindow *window)
 void
 nautilus_window_constructed (NautilusWindow *window)
 {
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	nautilus_window_set_initial_window_geometry (window);
 	nautilus_undo_manager_attach (window->application->undo_manager, G_OBJECT (window));
 }
@@ -539,7 +578,8 @@ static void
 nautilus_window_destroy (GtkObject *object)
 {
 	NautilusWindow *window;
-	
+	GtkWidget *widget;
+
 	window = NAUTILUS_WINDOW (object);
 
 	cancel_view_as_callback (window);
@@ -551,6 +591,13 @@ nautilus_window_destroy (GtkObject *object)
 		window->content_view = NULL;
 	}
 
+	if (window->new_content_view) {
+		widget = nautilus_view_get_widget (window->new_content_view);
+		gtk_widget_destroy (widget);
+		g_object_unref (window->new_content_view);
+		window->new_content_view = NULL;
+	}
+
 	GTK_OBJECT_CLASS (nautilus_window_parent_class)->destroy (object);
 }
 
@@ -594,6 +641,25 @@ nautilus_window_finalize (GObject *object)
 	G_OBJECT_CLASS (nautilus_window_parent_class)->finalize (object);
 }
 
+static GObject *
+nautilus_window_constructor (GType type,
+			     guint n_construct_properties,
+			     GObjectConstructParam *construct_params)
+{
+	GObject *object;
+	NautilusWindow *window;
+
+	object = (* G_OBJECT_CLASS (nautilus_window_parent_class)->constructor) (type,
+										 n_construct_properties,
+										 construct_params);
+
+	window = NAUTILUS_WINDOW (object);
+
+	nautilus_window_initialize_menus_constructed (window);
+
+	return object;
+}
+
 void
 nautilus_window_show_window (NautilusWindow *window)
 {
@@ -1006,13 +1072,14 @@ void
 nautilus_window_display_error (NautilusWindow *window, const char *error_msg)
 {
 	GtkWidget *dialog;
-	
+
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	dialog = gtk_message_dialog_new (GTK_WINDOW (window), 0, GTK_MESSAGE_ERROR,
 					 GTK_BUTTONS_OK, error_msg, NULL);
 	gtk_widget_show (dialog);
 }
 
-
 static char *
 real_get_title (NautilusWindow *window)
 {
@@ -1046,25 +1113,38 @@ nautilus_window_get_title (NautilusWindow *window)
 						  get_title, (window));
 }
 
-static void
+static gboolean
 real_set_title (NautilusWindow *window,
 		const char *title)
 {
+	gboolean changed;
 	char *copy;
 
-	g_free (window->details->title);
-        window->details->title = g_strdup (title);
+	changed = FALSE;
 
-        if (window->details->title [0] != '\0' && window->current_location_bookmark &&
+	if (eel_strcmp (title, window->details->title) != 0) {
+		changed = TRUE;
+
+		g_free (window->details->title);
+		window->details->title = g_strdup (title);
+	}
+
+        if (eel_strlen (window->details->title) > 0 && window->current_location_bookmark &&
             nautilus_bookmark_set_name (window->current_location_bookmark, window->details->title)) {
+		changed = TRUE;
+
                 /* Name of item in history list changed, tell listeners. */
                 nautilus_send_history_list_changed ();
         }
 
-	copy = g_strdup (window->details->title);
-	g_signal_emit_by_name (window, "title_changed",
-			       copy);
-	g_free (copy);
+	if (changed) {
+		copy = g_strdup (window->details->title);
+		g_signal_emit_by_name (window, "title_changed",
+				       copy);
+		g_free (copy);
+	}
+
+	return changed;
 }
 
 /* Sets window->details->title, and the actual GtkWindow title which
@@ -1073,10 +1153,7 @@ static void
 nautilus_window_set_title (NautilusWindow *window, 
 			   const char *title)
 {
-	if (window->details->title != NULL
-	    && strcmp (title, window->details->title) == 0) {
-		return;
-	}
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
                          set_title, (window, title));
@@ -1094,6 +1171,8 @@ void
 nautilus_window_update_title (NautilusWindow *window)
 {
 	char *title;
+
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 	
 	title = nautilus_window_get_title (window);
 	nautilus_window_set_title (window, title);
@@ -1113,6 +1192,8 @@ nautilus_window_update_icon (NautilusWindow *window)
 {
 	char *icon_name;
 
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
 	icon_name = EEL_CALL_METHOD_WITH_RETURN_VALUE (NAUTILUS_WINDOW_CLASS, window,
 						       get_icon_name, (window));
 
@@ -1165,10 +1246,12 @@ void
 nautilus_window_set_content_view_widget (NautilusWindow *window,
 					 NautilusView *frame)
 {
-	g_assert (NAUTILUS_IS_WINDOW (window));
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 	
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
                          set_content_view_widget, (window, frame));
+
+	nautilus_view_grab_focus (frame);
 }
 
 /**
@@ -1193,6 +1276,8 @@ nautilus_window_show (GtkWidget *widget)
 GtkUIManager *
 nautilus_window_get_ui_manager (NautilusWindow *window)
 {
+	g_return_val_if_fail (NAUTILUS_IS_WINDOW (window), NULL);
+
 	return window->details->ui_manager;
 }
 
@@ -1329,7 +1414,7 @@ real_add_current_location_to_history_list (NautilusWindow *window)
 void
 nautilus_window_add_current_location_to_history_list (NautilusWindow *window)
 {
-        g_assert (NAUTILUS_IS_WINDOW (window));
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
 	EEL_CALL_METHOD (NAUTILUS_WINDOW_CLASS, window,
                          add_current_location_to_history_list, (window));
@@ -1393,6 +1478,8 @@ nautilus_window_get_history (NautilusWindow *window)
 static NautilusWindowType
 nautilus_window_get_window_type (NautilusWindow *window)
 {
+	g_return_val_if_fail (NAUTILUS_IS_WINDOW (window), NAUTILUS_WINDOW_SPATIAL);
+
 	return NAUTILUS_WINDOW_GET_CLASS (window)->window_type;
 }
 
@@ -1463,8 +1550,9 @@ static void
 nautilus_window_class_init (NautilusWindowClass *class)
 {
 	GtkBindingSet *binding_set;
-	
+
 	G_OBJECT_CLASS (class)->finalize = nautilus_window_finalize;
+	G_OBJECT_CLASS (class)->constructor = nautilus_window_constructor;
 	G_OBJECT_CLASS (class)->get_property = nautilus_window_get_property;
 	G_OBJECT_CLASS (class)->set_property = nautilus_window_set_property;
 	GTK_OBJECT_CLASS (class)->destroy = nautilus_window_destroy;
@@ -1508,8 +1596,8 @@ nautilus_window_class_init (NautilusWindowClass *class)
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (NautilusWindowClass, prompt_for_location),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE, 0);
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1, G_TYPE_STRING);
 	
 	binding_set = gtk_binding_set_by_class (class);
 	gtk_binding_entry_add_signal (binding_set, GDK_BackSpace, 0,
@@ -1517,9 +1605,36 @@ nautilus_window_class_init (NautilusWindowClass *class)
 				      G_TYPE_BOOLEAN, FALSE);
 	gtk_binding_entry_add_signal (binding_set, GDK_F5, 0,
 				      "reload", 0);
-	gtk_binding_entry_add_signal (binding_set, GDK_d, GDK_MOD1_MASK,
-				      "prompt-for-location", 0);
+	gtk_binding_entry_add_signal (binding_set, GDK_slash, 0,
+				      "prompt-for-location", 1,
+				      G_TYPE_STRING, "/");
 
 	class->reload = nautilus_window_reload;
 	class->go_up = nautilus_window_go_up_signal;
+
+	/* Allow to set the colors of the extra view widgets */
+	gtk_rc_parse_string ("\n"
+			     "   style \"nautilus-extra-view-widgets-style-internal\"\n"
+			     "   {\n"
+			     "      bg[NORMAL] = \"" EXTRA_VIEW_WIDGETS_BACKGROUND "\"\n"
+			     "   }\n"
+			     "\n"
+			     "    widget \"*.nautilus-extra-view-widget\" style:rc \"nautilus-extra-view-widgets-style-internal\" \n"
+			     "\n");
+}
+
+/**
+ * nautilus_window_has_menubar_and_statusbar:
+ * @window: A #NautilusWindow
+ * 
+ * Queries whether the window should have a menubar and statusbar, based on the
+ * window_type from its class structure.
+ * 
+ * Return value: TRUE if the window should have a menubar and statusbar; FALSE
+ * otherwise.
+ **/
+gboolean
+nautilus_window_has_menubar_and_statusbar (NautilusWindow *window)
+{
+	return (nautilus_window_get_window_type (window) != NAUTILUS_WINDOW_DESKTOP);
 }
